/* SPDX-License-Identifier: GPL-2.0 */

#ifndef _DT_BINDINGS_CLK_LPASS_AONCC_SM8250_H
#define _DT_BINDINGS_CLK_LPASS_AONCC_SM8250_H

/* from AOCC */
#define LPASS_CDC_VA_MCLK				0
#define LPASS_CDC_TX_NPL				1
#define LPASS_CDC_TX_MCLK				2

#endif /* _DT_BINDINGS_CLK_LPASS_AONCC_SM8250_H */
